@@ -43,6 +43,7 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <pthread.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -90,6 +91,8 @@ static gboolean run_for_start = FALSE;
 static gchar* run_components = NULL;
 static gchar* login_password = NULL;
 static gboolean initialization_completed = FALSE;
+static gboolean sig_thread_valid = FALSE;
+static pthread_t sig_thread;
 
 static GOptionEntry option_entries[] = {
 	{ "foreground", 'f', 0, G_OPTION_ARG_NONE, &run_foreground, 
@@ -311,6 +314,14 @@ log_handler (const gchar *log_domain, GLogLevelFlags log_level,
     g_log_default_handler (log_domain, log_level, message, user_data); 
 }
 
+static void
+printerr_handler (const gchar *string)
+{
+	/* Print to syslog and stderr */
+	syslog (LOG_WARNING, "%s", string);
+	fprintf (stderr, "%s", string);
+}
+
 static void
 prepare_logging ()
 {
@@ -325,6 +336,7 @@ prepare_logging ()
     g_log_set_handler ("Gtk", flags, log_handler, NULL);
     g_log_set_handler ("Gnome", flags, log_handler, NULL);
     g_log_set_default_handler (log_handler, NULL);
+    g_set_printerr_handler (printerr_handler);
 }
 
 /* -----------------------------------------------------------------------------
@@ -370,7 +382,7 @@ signal_thread (gpointer user_data)
 static void
 setup_signal_handling (GMainLoop *loop)
 {
-	GError *error = NULL;
+	int res;
 
 	/*
 	 * Block these signals for this thread, and any threads
@@ -387,11 +399,12 @@ setup_signal_handling (GMainLoop *loop)
 	sigaddset (&signal_set, SIGTERM);
 	pthread_sigmask (SIG_BLOCK, &signal_set, NULL);
 
-	g_thread_create (signal_thread, loop, FALSE, &error);
-	if (error != NULL) {
+	res = pthread_create (&sig_thread, NULL, signal_thread, loop);
+	if (res == 0) {
+		sig_thread_valid = TRUE;
+	} else {
 		g_warning ("couldn't startup thread for signal handling: %s",
-		           error && error->message ? error->message : "");
-		g_clear_error (&error);
+		           g_strerror (res));
 	}
 }
 
@@ -404,7 +417,10 @@ gkr_daemon_quit (void)
 	 * starts the shutdown process.
 	 */
 
-	raise (SIGTERM);
+	if (sig_thread_valid)
+		pthread_kill (sig_thread, SIGTERM);
+	else
+		raise (SIGTERM);
 }
 
 static void
@@ -839,7 +855,7 @@ main (int argc, char *argv[])
 	 */
 	if (login_password) {
 		if (!gkr_keyring_login_unlock (login_password))
-			g_warning ("Failed to unlock login on startup");
+			g_message ("Failed to unlock login on startup");
 		egg_secure_strclear (login_password);
 	}
 	
