@@ -6,8 +6,33 @@
 
 #include <gtk/gtk.h>
 
+#include <unistd.h>
+#include <string.h>
+#include <errno.h>
+
+static void 
+chdir_base_dir (char* argv0)
+{
+	gchar *dir, *base;
+
+	dir = g_path_get_dirname (argv0);
+	if (chdir (dir) < 0)
+		g_warning ("couldn't change directory to: %s: %s", 
+		           dir, g_strerror (errno));
+	
+	base = g_path_get_basename (dir);
+	if (strcmp (base, ".libs") == 0) {
+		if (chdir ("..") < 0)
+			g_warning ("couldn't change directory to ..: %s",
+			           g_strerror (errno));
+	}
+
+	g_free (base);
+	g_free (dir);
+}
+
 static void
-test_details (void)
+test_details (const gchar *path)
 {
 	GcrCertificateDetailsWidget *details;
 	GcrCertificate *certificate;
@@ -15,8 +40,8 @@ test_details (void)
 	guchar *data;
 	gsize n_data;
 	
-	if (!g_file_get_contents ("test-data/der-certificate.crt", (gchar**)&data, &n_data, NULL))
-		g_assert_not_reached ();
+	if (!g_file_get_contents (path, (gchar**)&data, &n_data, NULL))
+		g_error ("couldn't read file: %s", path);
 	
 	certificate = gcr_simple_certificate_new (data, n_data);
 	g_assert (certificate);
@@ -41,6 +66,13 @@ int
 main(int argc, char *argv[])
 {
 	gtk_init (&argc, &argv);
-	test_details ();
+	
+	if (argc > 1) {
+		test_details (argv[1]);
+	} else {
+		chdir_base_dir (argv[0]);
+		test_details ("test-data/der-certificate.crt");
+	}
+	
 	return 0;
 }
