@@ -28,6 +28,21 @@
 
 #include <string.h>
 
+/**
+ * SECTION:gp11-slot
+ * @title: GP11Slot
+ * @short_description: Represents a PKCS11 slot that can contain a token.
+ * 
+ * A PKCS11 slot can contain a token. As an example, a slot might be a card reader, and the token
+ * the card. If the PKCS11 module is not a hardware driver, often the slot and token are equivalent.  
+ */
+
+/**
+ * GP11Slot:
+ * 
+ * Represents a PKCS11 slot.
+ */
+
 enum {
 	PROP_0,
 	PROP_MODULE,
@@ -187,10 +202,20 @@ gp11_slot_class_init (GP11SlotClass *klass)
 	gobject_class->dispose = gp11_slot_dispose;
 	gobject_class->finalize = gp11_slot_finalize;
 	
+	/**
+	 * GP11Slot:module:
+	 * 
+	 * The PKCS11 object that this slot is a part of.
+	 */
 	g_object_class_install_property (gobject_class, PROP_MODULE,
 		g_param_spec_object ("module", "Module", "PKCS11 Module",
 		                     GP11_TYPE_MODULE, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	/**
+	 * GP11Slot:handle:
+	 * 
+	 * The raw CK_SLOT_ID handle of this slot.
+	 */
 	g_object_class_install_property (gobject_class, PROP_HANDLE,
 		g_param_spec_ulong ("handle", "Handle", "PKCS11 Slot ID",
 		                   0, G_MAXULONG, 0, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
@@ -199,26 +224,26 @@ gp11_slot_class_init (GP11SlotClass *klass)
 }
 
 /* ----------------------------------------------------------------------------
- * INTERNAL AUTHENTICATION
+ * PUBLIC 
  */
 
-gboolean
-_gp11_slot_is_protected_auth_path (GP11Slot *self)
-{
-	GP11TokenInfo *info;
-	gboolean ret;
-
-	g_assert (GP11_IS_SLOT (self));
-
-	info = gp11_slot_get_token_info (self);
-	ret = (info && info->flags & CKF_PROTECTED_AUTHENTICATION_PATH);
-	gp11_token_info_free (info);
-	
-	return ret;
-}
-
-/* ----------------------------------------------------------------------------
- * PUBLIC 
+/**
+ * GP11SlotInfo:
+ * @slot_description: Description of the slot.
+ * @manufacturer_id: The manufacturer of this slot.
+ * @flags: Various PKCS11 flags that apply to this slot. 
+ * @hardware_version_major: The major version of the hardware.
+ * @hardware_version_minor: The minor version of the hardware.
+ * @firmware_version_major: The major version of the firmware.
+ * @firmware_version_minor: The minor version of the firmware.
+ * 
+ * Represents information about a PKCS11 slot.
+ * 
+ * This is analogous to a CK_SLOT_INFO structure, but the 
+ * strings are far more usable.
+ * 
+ * When you're done with this structure it should be released with 
+ * gp11_slot_info_free().
  */
 
 /**
@@ -237,6 +262,38 @@ gp11_slot_info_free (GP11SlotInfo *slot_info)
 	g_free (slot_info);
 }
 
+/**
+ * GP11TokenInfo:
+ * @label: The displayable token label.
+ * @manufacturer_id: The manufacturer of this slot.
+ * @model: The token model number as a string.
+ * @serial_number: The token serial number as a string.
+ * @flags: Various PKCS11 flags that apply to this token.
+ * @max_session_count: The maximum number of sessions allowed on this token.
+ * @session_count: The number of sessions open on this token.
+ * @max_rw_session_count: The maximum number of read/write sessions allowed on this token.
+ * @rw_session_count: The number of sessions open on this token.
+ * @max_pin_len: The maximum length of a PIN for locking this token.
+ * @min_pin_len: The minimum length of a PIN for locking this token.
+ * @total_public_memory: The total amount of memory on this token for storing public objects.
+ * @free_public_memory: The available amount of memory on this token for storing public objects.  
+ * @total_private_memory: The total amount of memory on this token for storing private objects.
+ * @free_private_memory: The available amount of memory on this token for storing private objects.  
+ * @hardware_version_major: The major version of the hardware.
+ * @hardware_version_minor: The minor version of the hardware.
+ * @firmware_version_major: The major version of the firmware.
+ * @firmware_version_minor: The minor version of the firmware.
+ * @utc_time: If the token has a hardware clock, this is set to the number of seconds since the epoch.
+ * 
+ * Represents information about a PKCS11 token.
+ * 
+ * This is analogous to a CK_TOKEN_INFO structure, but the 
+ * strings are far more usable.
+ * 
+ * When you're done with this structure it should be released with 
+ * gp11_token_info_free().
+ */
+
 /**
  * gp11_token_info_free:
  * @token_info: The token info to free, or NULL.
@@ -255,6 +312,20 @@ gp11_token_info_free (GP11TokenInfo *token_info)
 	g_free (token_info);
 }
 
+/**
+ * GP11MechanismInfo:
+ * @min_key_size: The minimum key size that can be used with this mechanism.
+ * @max_key_size: The maximum key size that can be used with this mechanism.
+ * @flags: Various PKCS11 flags that apply to this mechanism.
+ * 
+ * Represents information about a PKCS11 mechanism.
+ * 
+ * This is analogous to a CK_MECHANISM_INFO structure.
+ * 
+ * When you're done with this structure it should be released with 
+ * gp11_mechanism_info_free().
+ */
+
 /**
  * gp11_mechanism_info_free:
  * @mech_info: The mechanism info to free, or NULL.
@@ -269,6 +340,38 @@ gp11_mechanism_info_free (GP11MechanismInfo *mech_info)
 	g_free (mech_info);
 }
 
+/**
+ * GP11Mechanisms:
+ * 
+ * A set of GP11MechanismInfo structures.
+ */
+
+/**
+ * gp11_mechanisms_length:
+ * @a: A GP11Mechanisms set.
+ * 
+ * Get the number of GP11MechanismInfo in the set.
+ * 
+ * Returns: The number in the set.
+ */
+
+/**
+ * gp11_mechanisms_at:
+ * @a: A GP11Mechanisms set.
+ * @i: The index of a GP11MechanismInfo.
+ * 
+ * Get a specific GP11MechanismInfo in a the set.
+ * 
+ * Returns: The GP11MechanismInfo.
+ */
+
+/**
+ * gp11_mechanisms_free:
+ * @a: A GP11Mechanisms set.
+ * 
+ * Free a GP11Mechanisms set.
+ */
+
 /**
  * gp11_mechanisms_check:
  * @mechanisms: A list of mechanisms, perhaps retrieved from gp11_slot_get_mechanisms().
@@ -313,6 +416,57 @@ gp11_mechanisms_check (GP11Mechanisms *mechanisms, ...)
 	return found;
 }
 
+/**
+ * gp11_slot_equal:
+ * @slot1: A pointer to the first GP11Slot
+ * @slot2: A pointer to the second GP11Slot
+ * 
+ * Checks equality of two slots. Two GP11Slot objects can point to the same 
+ * underlying PKCS#11 slot.
+ * 
+ * Return value: TRUE if slot1 and slot2 are equal. FALSE if either is not a GP11Slot.
+ **/
+gboolean
+gp11_slot_equal (gconstpointer slot1, gconstpointer slot2)
+{
+	GP11SlotData *data1, *data2;
+
+	if (slot1 == slot2)
+		return TRUE;
+	if (!GP11_IS_SLOT (slot1) || !GP11_IS_SLOT (slot2))
+		return FALSE;
+	
+	data1 = GP11_SLOT_GET_DATA (slot1);
+	data2 = GP11_SLOT_GET_DATA (slot2);
+	
+	return data1->handle == data2->handle && 
+	       gp11_module_equal (data1->module, data2->module);
+}
+
+/**
+ * gp11_slot_hash:
+ * @slot: A pointer to a GP11Slot
+ * 
+ * Create a hash value for the GP11Slot. 
+ * 
+ * This function is intended for easily hashing a GP11Slot to add to 
+ * a GHashTable or similar data structure.
+ * 
+ * Return value: An integer that can be used as a hash value, or 0 if invalid.
+ **/
+guint
+gp11_slot_hash (gconstpointer slot)
+{
+	GP11SlotData *data;
+	
+	g_return_val_if_fail (GP11_IS_SLOT (slot), 0);
+
+	data = GP11_SLOT_GET_DATA (slot);
+	
+	return _gp11_ulong_hash (&data->handle) ^
+	       gp11_module_hash (data->module);
+}
+
 /**
  * gp11_slot_get_handle:
  * @self: The slot to get the handle of.
@@ -466,6 +620,7 @@ gp11_slot_get_token_info (GP11Slot *self)
 			tokeninfo->utc_time = -1;
 		else
 			tokeninfo->utc_time = timegm (&tm);
+		g_free (string);
 	} else {
 		tokeninfo->utc_time = -1;
 	}
@@ -576,6 +731,45 @@ gp11_slot_get_mechanism_info (GP11Slot *self, gulong mech_type)
 	return mechinfo;
 }
 
+/**
+ * gp11_slot_has_flags:
+ * @self: The GP11Slot object.
+ * @flags: The flags to check.
+ * 
+ * Check if the PKCS11 slot has the given flags.
+ * 
+ * Returns: Whether one or more flags exist.
+ */
+gboolean
+gp11_slot_has_flags (GP11Slot *self, gulong flags)
+{
+	CK_FUNCTION_LIST_PTR funcs;
+	GP11Module *module = NULL;
+	CK_TOKEN_INFO info;
+	CK_SLOT_ID handle;
+	CK_RV rv;
+	
+	g_return_val_if_fail (GP11_IS_SLOT (self), FALSE);
+
+	g_object_get (self, "module", &module, "handle", &handle, NULL);
+	g_return_val_if_fail (GP11_IS_MODULE (module), FALSE);
+	
+	funcs = gp11_module_get_functions (module);
+	g_return_val_if_fail (funcs, FALSE);
+	
+	memset (&info, 0, sizeof (info));
+	rv = (funcs->C_GetTokenInfo) (handle, &info);
+	
+	g_object_unref (module);
+	
+	if (rv != CKR_OK) {
+		g_warning ("couldn't get slot info: %s", gp11_message_from_rv (rv));
+		return FALSE;
+	}
+	
+	return (info.flags & flags) != 0;
+}
+
 #if UNIMPLEMENTED
 
 typedef struct InitToken {
@@ -760,7 +954,7 @@ gp11_slot_open_session_full (GP11Slot *self, gulong flags, GCancellable *cancell
 		args.slot = self;
 		args.flags = flags;
 		args.password = NULL;
-		args.auto_login = gp11_module_get_auto_authenticate (module);
+		args.auto_login = (gp11_module_get_auto_authenticate (module) & GP11_AUTHENTICATE_TOKENS) ? TRUE : FALSE;
 		args.session = 0;
 		
 		if (_gp11_call_sync (self, perform_open_session, complete_open_session, &args, cancellable, err))
@@ -809,7 +1003,7 @@ gp11_slot_open_session_async (GP11Slot *self, gulong flags, GCancellable *cancel
 	module = gp11_slot_get_module (self);
 	slot_id = gp11_slot_get_handle (self);
 	args->session = _gp11_module_pooled_session_handle (module, slot_id, flags);
-	args->auto_login = gp11_module_get_auto_authenticate (module);
+	args->auto_login = (gp11_module_get_auto_authenticate (module) & GP11_AUTHENTICATE_TOKENS) ? TRUE : FALSE;
 	g_object_unref (module);
 	
 	call = _gp11_call_async_ready (args, cancellable, callback, user_data);
