@@ -28,36 +28,36 @@
 #include "gnome-keyring-proto.h"
 #include "gnome-keyring-private.h"
 
-#include "common/gkr-buffer.h"
-#include "common/gkr-secure-memory.h"
+#include "egg/egg-buffer.h"
+#include "egg/egg-secure-memory.h"
 
 void 
-gkr_proto_go_secure (GkrBuffer *buffer)
+gkr_proto_go_secure (EggBuffer *buffer)
 {
-	gkr_buffer_set_allocator (buffer, gkr_secure_realloc);
+	egg_buffer_set_allocator (buffer, egg_secure_realloc);
 }
 
 void
-gkr_proto_add_time (GkrBuffer *buffer, time_t time)
+gkr_proto_add_time (EggBuffer *buffer, time_t time)
 {
 	guint64 val;
 
 	val = time;
-	gkr_buffer_add_uint32 (buffer, ((val >> 32) & 0xffffffff));
-	gkr_buffer_add_uint32 (buffer, (val & 0xffffffff));
+	egg_buffer_add_uint32 (buffer, ((val >> 32) & 0xffffffff));
+	egg_buffer_add_uint32 (buffer, (val & 0xffffffff));
 }
 
 gboolean
-gkr_proto_get_time (GkrBuffer *buffer, gsize offset, gsize *next_offset,
+gkr_proto_get_time (EggBuffer *buffer, gsize offset, gsize *next_offset,
                     time_t *time)
 {
 	guint32 a, b;
 	guint64 val;
 
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &a)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &a)) {
 		return FALSE;
 	}
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &b)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &b)) {
 		return FALSE;
 	}
 
@@ -70,7 +70,7 @@ gkr_proto_get_time (GkrBuffer *buffer, gsize offset, gsize *next_offset,
 }
 
 gboolean
-gkr_proto_add_utf8_secret (GkrBuffer *buffer, const char *str)
+gkr_proto_add_utf8_secret (EggBuffer *buffer, const char *str)
 {
 	/* Make sure this buffer is using non-pageable memory */	
 	gkr_proto_go_secure (buffer);
@@ -79,7 +79,7 @@ gkr_proto_add_utf8_secret (GkrBuffer *buffer, const char *str)
 }
 
 gboolean
-gkr_proto_add_utf8_string (GkrBuffer *buffer, const char *str)
+gkr_proto_add_utf8_string (EggBuffer *buffer, const char *str)
 {
 	gsize len;
 
@@ -93,11 +93,11 @@ gkr_proto_add_utf8_string (GkrBuffer *buffer, const char *str)
 		len = 0;
 	} 
 
-	return 	gkr_buffer_add_string (buffer, str);
+	return 	egg_buffer_add_string (buffer, str);
 }
 
 gboolean
-gkr_proto_get_bytes (GkrBuffer *buffer, gsize offset, gsize *next_offset,
+gkr_proto_get_bytes (EggBuffer *buffer, gsize offset, gsize *next_offset,
                      guchar *out, gsize n_bytes)
 {
 	if (buffer->len < n_bytes ||
@@ -112,29 +112,49 @@ gkr_proto_get_bytes (GkrBuffer *buffer, gsize offset, gsize *next_offset,
 }
 
 gboolean
-gkr_proto_get_utf8_string (GkrBuffer *buffer, gsize offset, gsize *next_offset,
-                           char **str_ret)
+gkr_proto_get_raw_secret (EggBuffer *buffer, gsize offset, gsize *next_offset,
+                          guchar **secret, gsize *n_secret)
 {
-	return gkr_proto_get_utf8_full (buffer, offset, next_offset, 
-	                                str_ret, (GkrBufferAllocator)g_realloc);
+	const guchar* ptr;
+	if (!egg_buffer_get_byte_array (buffer, offset, next_offset, &ptr, n_secret))
+		return FALSE;
+
+	if (ptr == NULL || *n_secret == 0) {
+		*secret = NULL;
+		*n_secret = 0;
+		return TRUE;
+	}
+
+	*secret = egg_secure_alloc (*n_secret + 1);
+	memcpy (*secret, ptr, *n_secret);
+	(*secret)[*n_secret] = 0;
+	return TRUE;
 }
 
 gboolean
-gkr_proto_get_utf8_secret (GkrBuffer *buffer, gsize offset, gsize *next_offset,
+gkr_proto_get_utf8_string (EggBuffer *buffer, gsize offset, gsize *next_offset,
                            char **str_ret)
 {
 	return gkr_proto_get_utf8_full (buffer, offset, next_offset, 
-	                                str_ret, gkr_secure_realloc);
+	                                str_ret, (EggBufferAllocator)g_realloc);
 }
 
 gboolean
-gkr_proto_get_utf8_full (GkrBuffer *buffer, gsize offset, gsize *next_offset,
-                         char **str_ret, GkrBufferAllocator allocator)
+gkr_proto_get_utf8_secret (EggBuffer *buffer, gsize offset, gsize *next_offset,
+                           char **str_ret)
+{
+	return gkr_proto_get_utf8_full (buffer, offset, next_offset, 
+	                                str_ret, egg_secure_realloc);
+}
+
+gboolean
+gkr_proto_get_utf8_full (EggBuffer *buffer, gsize offset, gsize *next_offset,
+                         char **str_ret, EggBufferAllocator allocator)
 {
 	gsize len;
 	char *str;
 	
-	if (!gkr_buffer_get_string (buffer, offset, &offset, &str, allocator))
+	if (!egg_buffer_get_string (buffer, offset, &offset, &str, allocator))
 		return FALSE;
 	len = str ? strlen (str) : 0;
 
@@ -157,14 +177,14 @@ gkr_proto_get_utf8_full (GkrBuffer *buffer, gsize offset, gsize *next_offset,
 }
 
 static gboolean
-gkr_proto_start_operation (GkrBuffer *buffer, GnomeKeyringOpCode op,
+gkr_proto_start_operation (EggBuffer *buffer, GnomeKeyringOpCode op,
                            gsize *op_start)
 {
 	gsize appname_pos;
 	const char *name;
 
 	appname_pos = buffer->len;
-	gkr_buffer_add_uint32 (buffer, 0);
+	egg_buffer_add_uint32 (buffer, 0);
 	
 	name = g_get_application_name ();
 	if (name != NULL && !g_utf8_validate (name, -1, NULL)) {
@@ -182,47 +202,47 @@ gkr_proto_start_operation (GkrBuffer *buffer, GnomeKeyringOpCode op,
 	}
 
 	/* backpatch application name size */
-	if (!gkr_buffer_set_uint32 (buffer, appname_pos, buffer->len)) {
+	if (!egg_buffer_set_uint32 (buffer, appname_pos, buffer->len)) {
 		return FALSE;
 	}
 
 	
 	/* Make space for packet size */
 	*op_start = buffer->len;
-	gkr_buffer_add_uint32 (buffer, 0);
-	gkr_buffer_add_uint32 (buffer, op);
+	egg_buffer_add_uint32 (buffer, 0);
+	egg_buffer_add_uint32 (buffer, op);
 	
 	return TRUE;
 }
 
 static gboolean
-gkr_proto_end_operation (GkrBuffer *buffer, gsize op_start)
+gkr_proto_end_operation (EggBuffer *buffer, gsize op_start)
 {
-	if (!gkr_buffer_set_uint32 (buffer, op_start, buffer->len - op_start)) {
+	if (!egg_buffer_set_uint32 (buffer, op_start, buffer->len - op_start)) {
 		return FALSE;
 	}
 	return TRUE;
 }
 
 gboolean
-gkr_proto_decode_packet_size (GkrBuffer *buffer, guint32 *size)
+gkr_proto_decode_packet_size (EggBuffer *buffer, guint32 *size)
 {
-	return gkr_buffer_get_uint32 (buffer, 0, NULL, size);
+	return egg_buffer_get_uint32 (buffer, 0, NULL, size);
 }
 
 gboolean
-gkr_proto_decode_packet_operation (GkrBuffer *buffer, GnomeKeyringOpCode *op)
+gkr_proto_decode_packet_operation (EggBuffer *buffer, GnomeKeyringOpCode *op)
 {
 	guint32 op_nr;
 	gboolean res;
 
-	res = gkr_buffer_get_uint32 (buffer, 4, NULL, &op_nr);
+	res = egg_buffer_get_uint32 (buffer, 4, NULL, &op_nr);
 	*op = op_nr;
 	return res;
 }
 
 gboolean
-gkr_proto_encode_op_only (GkrBuffer *buffer, GnomeKeyringOpCode op)
+gkr_proto_encode_op_only (EggBuffer *buffer, GnomeKeyringOpCode op)
 {
 	gsize op_start;
 
@@ -237,7 +257,7 @@ gkr_proto_encode_op_only (GkrBuffer *buffer, GnomeKeyringOpCode op)
 }
 
 gboolean
-gkr_proto_encode_op_string (GkrBuffer *buffer, GnomeKeyringOpCode op,
+gkr_proto_encode_op_string (EggBuffer *buffer, GnomeKeyringOpCode op,
                             const char *str)
 {
 	gsize op_start;
@@ -256,7 +276,7 @@ gkr_proto_encode_op_string (GkrBuffer *buffer, GnomeKeyringOpCode op,
 }
 
 gboolean
-gkr_proto_encode_op_string_int (GkrBuffer *buffer, GnomeKeyringOpCode op,
+gkr_proto_encode_op_string_int (EggBuffer *buffer, GnomeKeyringOpCode op,
                                 const char *str, guint32 val)
 {
 	gsize op_start;
@@ -267,7 +287,7 @@ gkr_proto_encode_op_string_int (GkrBuffer *buffer, GnomeKeyringOpCode op,
 	if (!gkr_proto_add_utf8_string (buffer, str)) {
 		return FALSE;
 	}
-	gkr_buffer_add_uint32 (buffer,	val);
+	egg_buffer_add_uint32 (buffer,	val);
 	if (!gkr_proto_end_operation (buffer,	op_start)) {
 		return FALSE;
 	}
@@ -276,7 +296,7 @@ gkr_proto_encode_op_string_int (GkrBuffer *buffer, GnomeKeyringOpCode op,
 }
 
 gboolean
-gkr_proto_encode_op_string_int_int (GkrBuffer *buffer, GnomeKeyringOpCode op,
+gkr_proto_encode_op_string_int_int (EggBuffer *buffer, GnomeKeyringOpCode op,
                                     const char *str, guint32 integer1,
                                     guint32 integer2)
 {
@@ -285,15 +305,15 @@ gkr_proto_encode_op_string_int_int (GkrBuffer *buffer, GnomeKeyringOpCode op,
 		return FALSE;
 	if (!gkr_proto_add_utf8_string (buffer, str))
 		return FALSE;
-	gkr_buffer_add_uint32 (buffer,	integer1);
-	gkr_buffer_add_uint32 (buffer,	integer2);
+	egg_buffer_add_uint32 (buffer,	integer1);
+	egg_buffer_add_uint32 (buffer,	integer2);
 	if (!gkr_proto_end_operation (buffer, op_start))
 		return FALSE;
 	return TRUE;
 }
 
 gboolean
-gkr_proto_encode_op_string_secret (GkrBuffer *buffer, GnomeKeyringOpCode op,
+gkr_proto_encode_op_string_secret (EggBuffer *buffer, GnomeKeyringOpCode op,
                                    const char *str1, const char *str2)
 {
 	gsize op_start;
@@ -318,7 +338,7 @@ gkr_proto_encode_op_string_secret (GkrBuffer *buffer, GnomeKeyringOpCode op,
 }
 
 gboolean
-gkr_proto_encode_op_string_secret_secret (GkrBuffer *buffer, GnomeKeyringOpCode op,
+gkr_proto_encode_op_string_secret_secret (EggBuffer *buffer, GnomeKeyringOpCode op,
                                           const char *str1, const char *str2,
                                           const char *str3)
 {
@@ -347,14 +367,14 @@ gkr_proto_encode_op_string_secret_secret (GkrBuffer *buffer, GnomeKeyringOpCode
 }
 
 gboolean
-gkr_proto_encode_find (GkrBuffer *buffer, GnomeKeyringItemType type,
+gkr_proto_encode_find (EggBuffer *buffer, GnomeKeyringItemType type,
                        GnomeKeyringAttributeList *attributes)
 {
 	gsize op_start;
 	
 	gkr_proto_start_operation (buffer, GNOME_KEYRING_OP_FIND, &op_start);
 
-	gkr_buffer_add_uint32 (buffer, type);
+	egg_buffer_add_uint32 (buffer, type);
 
 	if (!gkr_proto_add_attribute_list (buffer, attributes)) {
 		goto bail;
@@ -367,12 +387,12 @@ gkr_proto_encode_find (GkrBuffer *buffer, GnomeKeyringItemType type,
 	return TRUE;
 	
  bail:
- 	gkr_buffer_resize (buffer, op_start);
+ 	egg_buffer_resize (buffer, op_start);
 	return FALSE;
 }
 
 gboolean
-gkr_proto_encode_create_item (GkrBuffer *buffer, const char *keyring, 
+gkr_proto_encode_create_item (EggBuffer *buffer, const char *keyring, 
                               const char *display_name, 
                               GnomeKeyringAttributeList *attributes,
                               const char *secret, GnomeKeyringItemType type,
@@ -399,8 +419,8 @@ gkr_proto_encode_create_item (GkrBuffer *buffer, const char *keyring,
 	if (!gkr_proto_add_attribute_list (buffer, attributes)) {
 		return FALSE;
 	}
-	gkr_buffer_add_uint32 (buffer, type);
-	gkr_buffer_add_uint32 (buffer, update_if_exists);
+	egg_buffer_add_uint32 (buffer, type);
+	egg_buffer_add_uint32 (buffer, update_if_exists);
 	
 	if (!gkr_proto_end_operation (buffer,	op_start)) {
 		return FALSE;
@@ -410,7 +430,7 @@ gkr_proto_encode_create_item (GkrBuffer *buffer, const char *keyring,
 }
 
 gboolean
-gkr_proto_decode_create_item (GkrBuffer *buffer, char **keyring, char **display_name,
+gkr_proto_decode_create_item (EggBuffer *buffer, char **keyring, char **display_name,
                               GnomeKeyringAttributeList **attributes, char **secret,
                               GnomeKeyringItemType *type, gboolean *update_if_exists)
 {
@@ -451,14 +471,14 @@ gkr_proto_decode_create_item (GkrBuffer *buffer, char **keyring, char **display_
 	if (!gkr_proto_decode_attribute_list (buffer, offset, &offset, attributes)) {
 		goto bail;
 	}
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &val)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &val)) {
 		goto bail;
 	}
 	if (type != NULL) {
 		*type = val;
 	}
 
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &val)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &val)) {
 		goto bail;
 	}
 	if (update_if_exists != NULL) {
@@ -478,7 +498,7 @@ gkr_proto_decode_create_item (GkrBuffer *buffer, char **keyring, char **display_
 		g_free (*display_name);
 	}
 	if (secret != NULL) {
-		gkr_secure_strfree (*secret);
+		egg_secure_strfree (*secret);
 	}
 	return FALSE;
 	
@@ -486,7 +506,7 @@ gkr_proto_decode_create_item (GkrBuffer *buffer, char **keyring, char **display_
 
 
 gboolean
-gkr_proto_encode_set_attributes (GkrBuffer *buffer, const char *keyring,
+gkr_proto_encode_set_attributes (EggBuffer *buffer, const char *keyring,
                                  guint32 id, GnomeKeyringAttributeList *attributes)
 {
 	gsize op_start;
@@ -498,7 +518,7 @@ gkr_proto_encode_set_attributes (GkrBuffer *buffer, const char *keyring,
 	if (!gkr_proto_add_utf8_string (buffer, keyring)) {
 		return FALSE;
 	}
-	gkr_buffer_add_uint32 (buffer, id);
+	egg_buffer_add_uint32 (buffer, id);
 	
 	if (!gkr_proto_add_attribute_list (buffer, attributes)) {
 		return FALSE;
@@ -512,7 +532,7 @@ gkr_proto_encode_set_attributes (GkrBuffer *buffer, const char *keyring,
 }
 
 gboolean
-gkr_proto_encode_set_acl (GkrBuffer *buffer, const char *keyring,
+gkr_proto_encode_set_acl (EggBuffer *buffer, const char *keyring,
                           guint32 id, GList *acl)
 {
 	gsize op_start;
@@ -524,7 +544,7 @@ gkr_proto_encode_set_acl (GkrBuffer *buffer, const char *keyring,
 	if (!gkr_proto_add_utf8_string (buffer, keyring)) {
 		return FALSE;
 	}
-	gkr_buffer_add_uint32 (buffer, id);
+	egg_buffer_add_uint32 (buffer, id);
 	
 	if (!gkr_proto_add_acl (buffer, acl)) {
 		return FALSE;
@@ -539,7 +559,7 @@ gkr_proto_encode_set_acl (GkrBuffer *buffer, const char *keyring,
 
 
 gboolean
-gkr_proto_encode_set_item_info (GkrBuffer *buffer, const char *keyring,
+gkr_proto_encode_set_item_info (EggBuffer *buffer, const char *keyring,
                                 guint32 id, GnomeKeyringItemInfo *info)
 {
 	gsize op_start;
@@ -554,9 +574,9 @@ gkr_proto_encode_set_item_info (GkrBuffer *buffer, const char *keyring,
 	if (!gkr_proto_add_utf8_string (buffer, keyring)) {
 		return FALSE;
 	}
-	gkr_buffer_add_uint32 (buffer, id);
+	egg_buffer_add_uint32 (buffer, id);
 	
-	gkr_buffer_add_uint32 (buffer, info->type);
+	egg_buffer_add_uint32 (buffer, info->type);
 	if (!gkr_proto_add_utf8_string (buffer, info->display_name)) {
 		return FALSE;
 	}
@@ -572,7 +592,7 @@ gkr_proto_encode_set_item_info (GkrBuffer *buffer, const char *keyring,
 }
 
 gboolean
-gkr_proto_encode_set_keyring_info (GkrBuffer *buffer, const char *keyring,
+gkr_proto_encode_set_keyring_info (EggBuffer *buffer, const char *keyring,
                                    GnomeKeyringInfo *info)
 {
 	gsize op_start;
@@ -585,8 +605,8 @@ gkr_proto_encode_set_keyring_info (GkrBuffer *buffer, const char *keyring,
 		return FALSE;
 	}
 	
-	gkr_buffer_add_uint32 (buffer, info->lock_on_idle);
-	gkr_buffer_add_uint32 (buffer, info->lock_timeout);
+	egg_buffer_add_uint32 (buffer, info->lock_on_idle);
+	egg_buffer_add_uint32 (buffer, info->lock_timeout);
 
 	if (!gkr_proto_end_operation (buffer, op_start)) {
 		return FALSE;
@@ -595,9 +615,26 @@ gkr_proto_encode_set_keyring_info (GkrBuffer *buffer, const char *keyring,
 	return TRUE;
 }
 
+gboolean
+gkr_proto_encode_prepare_environment (EggBuffer *buffer, const gchar **environment)
+{
+	gsize op_start;
+	
+	if (!gkr_proto_start_operation (buffer, GNOME_KEYRING_OP_PREPARE_ENVIRONMENT,
+	                                &op_start))
+		return FALSE;
+		
+	if (!egg_buffer_add_stringv (buffer, environment))
+		return FALSE;
+
+	if (!gkr_proto_end_operation (buffer, op_start))
+		return FALSE;
+
+	return TRUE;
+}
 
 gboolean
-gkr_proto_decode_attribute_list (GkrBuffer *buffer, gsize offset, gsize *next_offset,
+gkr_proto_decode_attribute_list (EggBuffer *buffer, gsize offset, gsize *next_offset,
                                  GnomeKeyringAttributeList **attributes_out)
 {
 	guint32 list_size;
@@ -611,7 +648,7 @@ gkr_proto_decode_attribute_list (GkrBuffer *buffer, gsize offset, gsize *next_of
 
 	attributes = NULL;
 	
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &list_size)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &list_size)) {
 		goto bail;
 	}
 
@@ -620,7 +657,7 @@ gkr_proto_decode_attribute_list (GkrBuffer *buffer, gsize offset, gsize *next_of
 		if (!gkr_proto_get_utf8_string (buffer, offset, &offset, &name)) {
 			goto bail;
 		}
-		if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &type)) {
+		if (!egg_buffer_get_uint32 (buffer, offset, &offset, &type)) {
 			g_free (name);
 			goto bail;
 		}
@@ -636,7 +673,7 @@ gkr_proto_decode_attribute_list (GkrBuffer *buffer, gsize offset, gsize *next_of
 			g_array_append_val (attributes, attribute);
 			break;
 		case GNOME_KEYRING_ATTRIBUTE_TYPE_UINT32:
-			if (!gkr_buffer_get_uint32 (buffer, offset, 
+			if (!egg_buffer_get_uint32 (buffer, offset, 
 			                            &offset, &val)) {
 				g_free (name);
 				goto bail;
@@ -666,7 +703,7 @@ gkr_proto_decode_attribute_list (GkrBuffer *buffer, gsize offset, gsize *next_of
 }
 
 gboolean
-gkr_proto_decode_acl (GkrBuffer *buffer, gsize offset, gsize *next_offset,
+gkr_proto_decode_acl (EggBuffer *buffer, gsize offset, gsize *next_offset,
                       GList **acl_out)
 {
 	guint32 list_size;
@@ -680,13 +717,11 @@ gkr_proto_decode_acl (GkrBuffer *buffer, gsize offset, gsize *next_offset,
 
 	acl = NULL;
 
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &list_size)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &list_size)) {
 		goto bail;
 	}
 
 	for (i = 0; i < list_size; i++) {
-		ac = g_new0 (GnomeKeyringAccessControl, 1);
-		ref = g_new0 (GnomeKeyringApplicationRef, 1);
 		if (!gkr_proto_get_utf8_string (buffer,
 							  offset, &offset,
 							  &display_name)) {
@@ -699,13 +734,15 @@ gkr_proto_decode_acl (GkrBuffer *buffer, gsize offset, gsize *next_offset,
 			goto bail;
 		}
 
-		if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &types_allowed)) {
+		if (!egg_buffer_get_uint32 (buffer, offset, &offset, &types_allowed)) {
 			g_free (display_name);
 			g_free (pathname);
 			goto bail;
 		}
+		ref = g_new0 (GnomeKeyringApplicationRef, 1);
 		ref->display_name = display_name;
 		ref->pathname = pathname;
+		ac = g_new0 (GnomeKeyringAccessControl, 1);
 		ac->application = ref;
 		ac->types_allowed = types_allowed;
 		acl = g_list_append (acl, ac);
@@ -720,33 +757,33 @@ gkr_proto_decode_acl (GkrBuffer *buffer, gsize offset, gsize *next_offset,
 	return TRUE;
 	
  bail:
-	g_list_free (acl);
+	gnome_keyring_acl_free (acl);
 	return FALSE;
 }
 
 
 gboolean
-gkr_proto_add_attribute_list (GkrBuffer *buffer, GnomeKeyringAttributeList *attributes)
+gkr_proto_add_attribute_list (EggBuffer *buffer, GnomeKeyringAttributeList *attributes)
 {
 	int i;
 	GnomeKeyringAttribute *array;
 
 	/* Null attributes = empty attribute array */
 	if (!attributes) {
-		gkr_buffer_add_uint32 (buffer, 0);
+		egg_buffer_add_uint32 (buffer, 0);
 		return TRUE;
 	}
 		
 	array = (GnomeKeyringAttribute *)attributes->data;
 
 	i = 0;
-	gkr_buffer_add_uint32 (buffer, attributes->len);
+	egg_buffer_add_uint32 (buffer, attributes->len);
 
 	for (i = 0; i < attributes->len; i++) {
 		if (!gkr_proto_add_utf8_string (buffer, array[i].name)) {
 			return FALSE;
 		}
-		gkr_buffer_add_uint32 (buffer, array[i].type);
+		egg_buffer_add_uint32 (buffer, array[i].type);
 		switch (array[i].type) {
 		case GNOME_KEYRING_ATTRIBUTE_TYPE_STRING:
 			if (!gkr_proto_add_utf8_string (buffer, array[i].value.string)) {
@@ -754,7 +791,7 @@ gkr_proto_add_attribute_list (GkrBuffer *buffer, GnomeKeyringAttributeList *attr
 			}
 			break;
 		case GNOME_KEYRING_ATTRIBUTE_TYPE_UINT32:
-			gkr_buffer_add_uint32 (buffer, array[i].value.integer);
+			egg_buffer_add_uint32 (buffer, array[i].value.integer);
 			break;
 		default:
 			g_assert_not_reached ();
@@ -765,7 +802,7 @@ gkr_proto_add_attribute_list (GkrBuffer *buffer, GnomeKeyringAttributeList *attr
 }
 
 gboolean
-gkr_proto_add_acl (GkrBuffer *buffer, GList *acl)
+gkr_proto_add_acl (EggBuffer *buffer, GList *acl)
 {
 	int length;
 	GnomeKeyringAccessControl *ac;
@@ -773,7 +810,7 @@ gkr_proto_add_acl (GkrBuffer *buffer, GList *acl)
 
 	length = g_list_length (acl);
 
-	gkr_buffer_add_uint32 (buffer, length);
+	egg_buffer_add_uint32 (buffer, length);
 
 	for (tmp = acl; tmp != NULL; tmp = tmp->next) {
 		ac = (GnomeKeyringAccessControl *)tmp->data;
@@ -783,7 +820,7 @@ gkr_proto_add_acl (GkrBuffer *buffer, GList *acl)
 		if (!gkr_proto_add_utf8_string (buffer, ac->application->pathname)) {
 			return FALSE;
 		}
-		gkr_buffer_add_uint32 (buffer, ac->types_allowed);
+		egg_buffer_add_uint32 (buffer, ac->types_allowed);
 	}
 
 	return TRUE;
@@ -792,14 +829,14 @@ gkr_proto_add_acl (GkrBuffer *buffer, GList *acl)
 
 
 gboolean
-gkr_proto_decode_result_reply (GkrBuffer *buffer, GnomeKeyringResult *result)
+gkr_proto_decode_result_reply (EggBuffer *buffer, GnomeKeyringResult *result)
 {
 	gsize offset;
 	guint32 res;
 
 	offset = 4;
 
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &res)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &res)) {
 		return FALSE;
 	}
 	*result = res;
@@ -808,7 +845,7 @@ gkr_proto_decode_result_reply (GkrBuffer *buffer, GnomeKeyringResult *result)
 }
 
 gboolean
-gkr_proto_decode_result_string_reply (GkrBuffer *buffer, GnomeKeyringResult *result,
+gkr_proto_decode_result_string_reply (EggBuffer *buffer, GnomeKeyringResult *result,
                                       char **str)
 {
 	gsize offset;
@@ -816,7 +853,7 @@ gkr_proto_decode_result_string_reply (GkrBuffer *buffer, GnomeKeyringResult *res
 
 	offset = 4;
 
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &res)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &res)) {
 		return FALSE;
 	}
 	*result = res;
@@ -828,7 +865,7 @@ gkr_proto_decode_result_string_reply (GkrBuffer *buffer, GnomeKeyringResult *res
 }
 
 gboolean
-gkr_proto_decode_result_string_list_reply (GkrBuffer *buffer, GnomeKeyringResult *result,
+gkr_proto_decode_result_string_list_reply (EggBuffer *buffer, GnomeKeyringResult *result,
                                            GList **list)
 {
 	gsize offset;
@@ -840,11 +877,11 @@ gkr_proto_decode_result_string_list_reply (GkrBuffer *buffer, GnomeKeyringResult
 	offset = 4;
 	names = NULL;
 	
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &res)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &res)) {
 		return FALSE;
 	}
 
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &list_size)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &list_size)) {
 		goto bail;
 	}
 	
@@ -867,7 +904,7 @@ gkr_proto_decode_result_string_list_reply (GkrBuffer *buffer, GnomeKeyringResult
 }
 
 gboolean
-gkr_proto_decode_find_reply (GkrBuffer *buffer, GnomeKeyringResult *result,
+gkr_proto_decode_find_reply (EggBuffer *buffer, GnomeKeyringResult *result,
                              GList **list_out)
 {
 	GList *list;
@@ -878,7 +915,7 @@ gkr_proto_decode_find_reply (GkrBuffer *buffer, GnomeKeyringResult *result,
 	offset = 4;
 
 	*list_out = NULL;
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &res)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &res)) {
 		return FALSE;
 	}
 	*result = res;
@@ -895,7 +932,7 @@ gkr_proto_decode_find_reply (GkrBuffer *buffer, GnomeKeyringResult *result,
 		                                &found->keyring)) {
 			goto bail;
 		}
-		if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &found->item_id)) {
+		if (!egg_buffer_get_uint32 (buffer, offset, &offset, &found->item_id)) {
 			goto bail;
 		}
 		if (!gkr_proto_get_utf8_secret (buffer, offset, &offset,
@@ -917,7 +954,7 @@ gkr_proto_decode_find_reply (GkrBuffer *buffer, GnomeKeyringResult *result,
 }
 
 gboolean
-gkr_proto_decode_find (GkrBuffer *buffer, GnomeKeyringItemType *type,
+gkr_proto_decode_find (EggBuffer *buffer, GnomeKeyringItemType *type,
                        GnomeKeyringAttributeList **attributes)
 {
 	gsize offset;
@@ -932,7 +969,7 @@ gkr_proto_decode_find (GkrBuffer *buffer, GnomeKeyringItemType *type,
 	}
 
 	offset = 8;
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &t)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &t)) {
 		return FALSE;
 	}
 	*type = t;
@@ -940,7 +977,7 @@ gkr_proto_decode_find (GkrBuffer *buffer, GnomeKeyringItemType *type,
 }
 
 gboolean
-gkr_proto_decode_op_string (GkrBuffer *buffer, GnomeKeyringOpCode *op_out,
+gkr_proto_decode_op_string (EggBuffer *buffer, GnomeKeyringOpCode *op_out,
                             char **str1)
 {
 	gsize offset;
@@ -966,7 +1003,7 @@ gkr_proto_decode_op_string (GkrBuffer *buffer, GnomeKeyringOpCode *op_out,
 }
 
 gboolean
-gkr_proto_decode_op_string_int (GkrBuffer *buffer, GnomeKeyringOpCode *op_out,
+gkr_proto_decode_op_string_int (EggBuffer *buffer, GnomeKeyringOpCode *op_out,
                                 char **str1, guint32 *val)
 {
 	gsize offset;
@@ -982,7 +1019,7 @@ gkr_proto_decode_op_string_int (GkrBuffer *buffer, GnomeKeyringOpCode *op_out,
 		goto bail;
 	}
 
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, val)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, val)) {
 		goto bail;
 	}
 	
@@ -996,7 +1033,7 @@ gkr_proto_decode_op_string_int (GkrBuffer *buffer, GnomeKeyringOpCode *op_out,
 }
 
 gboolean
-gkr_proto_decode_get_item_info (GkrBuffer *buffer, GnomeKeyringOpCode *op_out,
+gkr_proto_decode_get_item_info (EggBuffer *buffer, GnomeKeyringOpCode *op_out,
                                 char **keyring, guint32 *item_id, guint32 *flags)
 {
 	gsize offset = 8;
@@ -1005,11 +1042,11 @@ gkr_proto_decode_get_item_info (GkrBuffer *buffer, GnomeKeyringOpCode *op_out,
 		return FALSE;
 	if (!gkr_proto_get_utf8_string (buffer, offset, &offset, keyring))
 		goto bail;
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, item_id))
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, item_id))
 		goto bail;
 	if (*op_out == GNOME_KEYRING_OP_GET_ITEM_INFO_FULL) {
 		/* Pull in lookup flags/parts, find out which ones */
-		if (!gkr_buffer_get_uint32 (buffer, offset, &offset, flags))
+		if (!egg_buffer_get_uint32 (buffer, offset, &offset, flags))
 			goto bail;
 	} else {
 		/* All parts of the item by default */
@@ -1024,7 +1061,7 @@ gkr_proto_decode_get_item_info (GkrBuffer *buffer, GnomeKeyringOpCode *op_out,
 }
 
 gboolean
-gkr_proto_decode_op_string_secret (GkrBuffer *buffer, GnomeKeyringOpCode *op_out,
+gkr_proto_decode_op_string_secret (EggBuffer *buffer, GnomeKeyringOpCode *op_out,
                                    char **str1, char **str2)
 {
 	gsize offset;
@@ -1060,7 +1097,7 @@ gkr_proto_decode_op_string_secret (GkrBuffer *buffer, GnomeKeyringOpCode *op_out
 }
 
 gboolean
-gkr_proto_decode_op_string_secret_secret (GkrBuffer *buffer, GnomeKeyringOpCode *op_out,
+gkr_proto_decode_op_string_secret_secret (EggBuffer *buffer, GnomeKeyringOpCode *op_out,
                                           char **str1, char **str2, char **str3)
 {
 	gsize offset;
@@ -1107,14 +1144,14 @@ gkr_proto_decode_op_string_secret_secret (GkrBuffer *buffer, GnomeKeyringOpCode
 
 
 gboolean
-gkr_proto_decode_get_attributes_reply (GkrBuffer *buffer, GnomeKeyringResult *result,
+gkr_proto_decode_get_attributes_reply (EggBuffer *buffer, GnomeKeyringResult *result,
                                        GnomeKeyringAttributeList **attributes)
 {
 	gsize offset;
 	guint32 res;
 
 	offset = 4;
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &res)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &res)) {
 		return FALSE;
 	}
 	*attributes = NULL;
@@ -1130,14 +1167,14 @@ gkr_proto_decode_get_attributes_reply (GkrBuffer *buffer, GnomeKeyringResult *re
 }
 
 gboolean
-gkr_proto_decode_get_acl_reply (GkrBuffer *buffer, GnomeKeyringResult *result,
+gkr_proto_decode_get_acl_reply (EggBuffer *buffer, GnomeKeyringResult *result,
                                 GList **acl)
 {
 	gsize offset;
 	guint32 res;
 
 	offset = 4;
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &res)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &res)) {
 		return FALSE;
 	}
 	*acl = NULL;
@@ -1154,7 +1191,7 @@ gkr_proto_decode_get_acl_reply (GkrBuffer *buffer, GnomeKeyringResult *result,
 
 
 gboolean
-gkr_proto_decode_get_item_info_reply (GkrBuffer *buffer, GnomeKeyringResult *result,
+gkr_proto_decode_get_item_info_reply (EggBuffer *buffer, GnomeKeyringResult *result,
                                       GnomeKeyringItemInfo      **info_out)
 {
 	gsize offset;
@@ -1167,11 +1204,11 @@ gkr_proto_decode_get_item_info_reply (GkrBuffer *buffer, GnomeKeyringResult *res
 	info = NULL;
 	
 	offset = 4;
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &res)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &res)) {
 		return FALSE;
 	}
 	if (res == GNOME_KEYRING_RESULT_OK) {
-		if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &type)) {
+		if (!egg_buffer_get_uint32 (buffer, offset, &offset, &type)) {
 			return FALSE;
 		}
 		if (!gkr_proto_get_utf8_string (buffer, offset, &offset, &name)) {
@@ -1184,12 +1221,12 @@ gkr_proto_decode_get_item_info_reply (GkrBuffer *buffer, GnomeKeyringResult *res
 		
 		if (!gkr_proto_get_time (buffer, offset, &offset, &mtime)) {
 			g_free (name);
-			gkr_secure_strfree (secret);
+			egg_secure_strfree (secret);
 			return FALSE;
 		}
 		if (!gkr_proto_get_time (buffer, offset, &offset, &ctime)) {
 			g_free (name);
-			gkr_secure_strfree (secret);
+			egg_secure_strfree (secret);
 			return FALSE;
 		}
 		
@@ -1208,7 +1245,7 @@ gkr_proto_decode_get_item_info_reply (GkrBuffer *buffer, GnomeKeyringResult *res
 }
 
 gboolean
-gkr_proto_decode_get_keyring_info_reply (GkrBuffer *buffer, GnomeKeyringResult *result,
+gkr_proto_decode_get_keyring_info_reply (EggBuffer *buffer, GnomeKeyringResult *result,
                                          GnomeKeyringInfo **info_out)
 {
 	gsize offset;
@@ -1220,15 +1257,15 @@ gkr_proto_decode_get_keyring_info_reply (GkrBuffer *buffer, GnomeKeyringResult *
 	info = NULL;
 	
 	offset = 4;
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &res)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &res)) {
 		return FALSE;
 	}
 	if (res == GNOME_KEYRING_RESULT_OK) {
-		if (!gkr_buffer_get_uint32 (buffer, offset, &offset,
+		if (!egg_buffer_get_uint32 (buffer, offset, &offset,
 		                            &lock_on_idle)) {
 			return FALSE;
 		}
-		if (!gkr_buffer_get_uint32 (buffer, offset, &offset,
+		if (!egg_buffer_get_uint32 (buffer, offset, &offset,
 		                            &lock_timeout)) {
 			return FALSE;
 		}
@@ -1238,7 +1275,7 @@ gkr_proto_decode_get_keyring_info_reply (GkrBuffer *buffer, GnomeKeyringResult *
 		if (!gkr_proto_get_time (buffer, offset, &offset, &ctime)) {
 			return FALSE;
 		}
-		if (!gkr_buffer_get_uint32 (buffer, offset, &offset,
+		if (!egg_buffer_get_uint32 (buffer, offset, &offset,
 		                            &is_locked)) {
 			return FALSE;
 		}
@@ -1257,7 +1294,7 @@ gkr_proto_decode_get_keyring_info_reply (GkrBuffer *buffer, GnomeKeyringResult *
 }
 
 gboolean
-gkr_proto_decode_set_item_info (GkrBuffer *buffer, char **keyring, guint32 *item_id,
+gkr_proto_decode_set_item_info (EggBuffer *buffer, char **keyring, guint32 *item_id,
                                 GnomeKeyringItemType *type, char **display_name,
                                 char **secret)
 {
@@ -1279,10 +1316,10 @@ gkr_proto_decode_set_item_info (GkrBuffer *buffer, char **keyring, guint32 *item
 	if (!gkr_proto_get_utf8_string (buffer, offset, &offset, keyring)) {
 		goto bail;
 	}
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, item_id)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, item_id)) {
 		goto bail;
 	}
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &typeint)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &typeint)) {
 		goto bail;
 	}
 	*type = typeint;
@@ -1299,12 +1336,12 @@ gkr_proto_decode_set_item_info (GkrBuffer *buffer, char **keyring, guint32 *item
  bail:
 	g_free (*keyring);
 	g_free (*display_name);
-	gkr_secure_strfree (*secret);
+	egg_secure_strfree (*secret);
 	return FALSE;
 }
 
 gboolean
-gkr_proto_decode_set_keyring_info (GkrBuffer *buffer, char **keyring,
+gkr_proto_decode_set_keyring_info (EggBuffer *buffer, char **keyring,
                                    gboolean *lock_on_idle, guint32 *lock_timeout)
 
 {
@@ -1324,12 +1361,12 @@ gkr_proto_decode_set_keyring_info (GkrBuffer *buffer, char **keyring,
 	if (!gkr_proto_get_utf8_string (buffer, offset, &offset, keyring)) {
 		goto bail;
 	}
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &lock_int)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &lock_int)) {
 		goto bail;
 	}
 	*lock_on_idle = lock_int;
 
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, lock_timeout)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, lock_timeout)) {
 		goto bail;
 	}
 
@@ -1341,7 +1378,7 @@ gkr_proto_decode_set_keyring_info (GkrBuffer *buffer, char **keyring,
 }
 
 gboolean
-gkr_proto_decode_set_attributes (GkrBuffer *buffer, char **keyring,
+gkr_proto_decode_set_attributes (EggBuffer *buffer, char **keyring,
                                  guint32 *item_id, GnomeKeyringAttributeList **attributes)
 {
 	gsize offset;
@@ -1360,7 +1397,7 @@ gkr_proto_decode_set_attributes (GkrBuffer *buffer, char **keyring,
 	if (!gkr_proto_get_utf8_string (buffer, offset, &offset, keyring)) {
 		goto bail;
 	}
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, item_id)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, item_id)) {
 		goto bail;
 	}
 	
@@ -1377,7 +1414,7 @@ gkr_proto_decode_set_attributes (GkrBuffer *buffer, char **keyring,
 
 
 gboolean
-gkr_proto_decode_set_acl (GkrBuffer *buffer, char **keyring, guint32 *item_id,
+gkr_proto_decode_set_acl (EggBuffer *buffer, char **keyring, guint32 *item_id,
                           GList  **acl)
 {
 	gsize offset;
@@ -1396,7 +1433,7 @@ gkr_proto_decode_set_acl (GkrBuffer *buffer, char **keyring, guint32 *item_id,
 	if (!gkr_proto_get_utf8_string (buffer, offset, &offset, keyring)) {
 		goto bail;
 	}
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, item_id)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, item_id)) {
 		goto bail;
 	}
 	
@@ -1411,10 +1448,48 @@ gkr_proto_decode_set_acl (GkrBuffer *buffer, char **keyring, guint32 *item_id,
 	return FALSE;
 }
 
+gboolean
+gkr_proto_decode_prepare_environment (EggBuffer *buffer, gchar ***environment)
+{
+	GnomeKeyringOpCode op;
+	gsize offset;
+	
+	if (!gkr_proto_decode_packet_operation (buffer, &op))
+		return FALSE;
+	if (op != GNOME_KEYRING_OP_PREPARE_ENVIRONMENT)
+		return FALSE;
+		
+	offset = 8;
+	
+	if (!egg_buffer_get_stringv (buffer, offset, &offset, environment, g_realloc))
+		return FALSE; 
+	
+	return TRUE;
+}
 
+gboolean 
+gkr_proto_decode_prepare_environment_reply (EggBuffer *buffer, GnomeKeyringResult *result,
+                                            char ***environment)
+{
+	gsize offset;
+	guint32 res;
+
+	offset = 4;
+
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &res))
+		return FALSE;
+	*result = res;
+
+	if (res == GNOME_KEYRING_RESULT_OK) {
+		if (!egg_buffer_get_stringv (buffer, offset, &offset, environment, g_realloc))
+			return FALSE; 
+	}		
+	
+	return TRUE;
+}
 
 gboolean
-gkr_proto_decode_result_int_list_reply (GkrBuffer *buffer, GnomeKeyringResult *result,
+gkr_proto_decode_result_int_list_reply (EggBuffer *buffer, GnomeKeyringResult *result,
                                         GList **list)
 {
 	gsize offset;
@@ -1424,17 +1499,17 @@ gkr_proto_decode_result_int_list_reply (GkrBuffer *buffer, GnomeKeyringResult *r
 
 	offset = 4;
 
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &res)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &res)) {
 		return FALSE;
 	}
 	*result = res;
 	
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &len)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &len)) {
 		return FALSE;
 	}
 	
 	for (i = 0; i < len; i++) {
-		if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &id)) {
+		if (!egg_buffer_get_uint32 (buffer, offset, &offset, &id)) {
 			g_list_free (*list);
 			*list = NULL;
 			return FALSE;
@@ -1447,7 +1522,7 @@ gkr_proto_decode_result_int_list_reply (GkrBuffer *buffer, GnomeKeyringResult *r
 }
 
 gboolean
-gkr_proto_decode_result_integer_reply (GkrBuffer *buffer, GnomeKeyringResult *result,
+gkr_proto_decode_result_integer_reply (EggBuffer *buffer, GnomeKeyringResult *result,
                                        guint32 *integer)
 {
 	gsize offset;
@@ -1455,10 +1530,10 @@ gkr_proto_decode_result_integer_reply (GkrBuffer *buffer, GnomeKeyringResult *re
 
 	offset = 4;
 
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &res)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &res)) {
 		return FALSE;
 	}
-	if (!gkr_buffer_get_uint32 (buffer, offset, &offset, &val)) {
+	if (!egg_buffer_get_uint32 (buffer, offset, &offset, &val)) {
 		return FALSE;
 	}
 	
